-----------------------------------------------------------------------------------------------
-- Client Lua Script for RealmWealth
-- Copyright (c) NCsoft. All rights reserved
-----------------------------------------------------------------------------------------------
 
require "Window"
require "ChatSystemLib"
require "GameLib"
require "Unit"
 
-----------------------------------------------------------------------------------------------
-- RealmWealth Module Definition
-----------------------------------------------------------------------------------------------
local RealmWealth = {} 
 
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
-- e.g. local kiExampleVariableMax = 999
 
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function RealmWealth:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    -- initialize variables here
	self.realms = {}
	
	self.HUDSettings = {}
	self.HUDSettings["show"] = true
	self.HUDSettings["show_text"] = true
	
	self.realmtotal = 0
	self.version = "1.4"
	self.characterFaction = 0;
	
    return o
end

function RealmWealth:Init()
	local bHasConfigureFunction = false
	local strConfigureButtonText = ""
	local tDependencies = {
		-- "UnitOrPackageName",
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)

end
 

-----------------------------------------------------------------------------------------------
-- RealmWealth OnLoad
-----------------------------------------------------------------------------------------------
function RealmWealth:OnLoad()
    -- load our form file
	self.xmlDoc = XmlDoc.CreateFromFile("RealmWealth.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

-----------------------------------------------------------------------------------------------
-- RealmWealth OnDocLoaded
-----------------------------------------------------------------------------------------------
function RealmWealth:OnDocLoaded()

	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
	    self.wndMain = Apollo.LoadForm(self.xmlDoc, "RealmWealthForm", nil, self)
		if self.wndMain == nil then
			Apollo.AddAddonErrorText(self, "Could not load the main window for some reason.")
			return
		end
		
		self.wndHUD = Apollo.LoadForm(self.xmlDoc, "RealmWealthUI", nil, self)
		if self.wndHUD == nil then
			Apollo.AddAddonErrorText(self, "Could not load the ui window for some reason.")
			return
		end
		
	    self.wndMain:Show(false, true)
		self.wndHUD:Show(false, true)
		
		self.wndMain:FindChild("Title"):SetText("RealmWealth v." .. self.version)
		
		-- if the xmlDoc is no longer needed, you should set it to nil
		-- self.xmlDoc = nil
		
		-- Register handlers for events, slash commands and timer, etc.
		-- e.g. Apollo.RegisterEventHandler("KeyDown", "OnKeyDown", self)
		Apollo.RegisterSlashCommand("wealth", "OnRealmWealthOn", self)
		Apollo.RegisterSlashCommand("realmwealth", "OnRealmWealthOn", self)
	
		Apollo.RegisterEventHandler("PlayerCurrencyChanged","OnPlayerCurrencyChanged", self)
		Apollo.RegisterTimerHandler("RWWaitForCharacterTimer","DoneLoading", self)
		Apollo.CreateTimer("RWWaitForCharacterTimer",1.0,false)
		
		
		-- Do additional Addon initialization here
		
		if self.realms[GameLib.GetRealmName()] == nil then
			self.realms[GameLib.GetRealmName()] = {}
		end
		
		self.wndMain:FindChild("Button_ShowHUD"):SetCheck(self.HUDSettings["show"])
		
		if self.HUDSettings["show"] == true then
			-- self:UpdateRealmWealthTotal()
			self.wndHUD:Invoke()
			
			-- set hud window text visibility
			if self.HUDSettings["show_text"] == true then
				self.wndHUD:SetText("Realm Wealth:")
				self.wndMain:FindChild("Button_ShowHUDTitle"):SetCheck(true)
			else
				self.wndHUD:SetText("")
				self.wndMain:FindChild("Button_ShowHUDTitle"):SetCheck(false)
			end
			
			self.wndMain:FindChild("Button_ShowHUD"):SetCheck(true)
		end
		
		
		if self.HUDSettings["position"] ~= nil then
			self.wndHUD:Move(self.HUDSettings["position"].left, self.HUDSettings["position"].top, self.wndHUD:GetWidth(), self.wndHUD:GetHeight())
		end
	end
end

-----------------------------------------------------------------------------------------------
-- RealmWealth Functions
-----------------------------------------------------------------------------------------------
-- Define general functions here

-- on SlashCommand "/wealth"
function RealmWealth:OnRealmWealthOn()
		
	self.characterFaction = GameLib.GetPlayerUnit():GetFaction()
	local faction = "Exiles"
	if self.characterFaction ~= Unit.CodeEnumFaction.ExilesPlayer then
		faction = "Dominion"
	end

	-- update most recent self wealth
	if self.realms == nil then self.realms = {} end
	if self.realms[GameLib.GetRealmName()] == nil then
		self.realms[GameLib.GetRealmName()] = {}
	end
	
	self:UpdateCurrentCharacterWealth()
	
	self:UpdateRealmWealthTotal()
	
	if GameLib.GetPlayerUnit():GetFaction() == Unit.CodeEnumFaction.ExilesPlayer then
		self.wndMain:FindChild("TitleFactionWealth"):SetText("Exiles:")
		self.wndMain:FindChild("TitleOtherFactionWealth"):SetText("Dominion:")
	else
		self.wndMain:FindChild("TitleFactionWealth"):SetText("Dominion:")
		self.wndMain:FindChild("TitleOtherFactionWealth"):SetText("Exiles:")
	end
	
	-- ChatSystemLib.PostOnChannel( ChatSystemLib.ChatChannel_System, "TOTAL REALM WEALTH: (" .. tostring(count) .. " characters)", "" )
	-- ChatSystemLib.PostOnChannel( ChatSystemLib.ChatChannel_System, text, "" )
	
	-- show settings panel
	local noteText = "Realm Wealth for " .. GameLib.GetRealmName()
    --local noteText = "Total Realm Wealth for " .. GameLib.GetRealmName() .. " " .. faction
	self.wndMain:FindChild("TitleRealmTop"):SetText(noteText)
	self.wndMain:Invoke() -- show the window
	
end

-- Wait for the character to be loaded before we try to count totals.
function RealmWealth:DoneLoading()
	if GameLib.GetPlayerUnit() == nil then
		-- ChatSystemLib.PostOnChannel( ChatSystemLib.ChatChannel_System, "waiting", "" )
		Apollo.CreateTimer("RWWaitForCharacterTimer",1.0,false)
		return	
	end
	-- ChatSystemLib.PostOnChannel( ChatSystemLib.ChatChannel_System, "done waiting", "" )
	self:UpdateCurrentCharacterWealth()
	self:UpdateRealmWealthTotal()
end

-- saves the player's current wealth information
function RealmWealth:UpdateCurrentCharacterWealth()
	local character = {}
	character["wealth"] = GameLib.GetPlayerCurrency():GetAmount()
	character["faction"] = GameLib.GetPlayerUnit():GetFaction()

	self.realms[GameLib.GetRealmName()][GameLib.GetPlayerUnit():GetName()] = character
end

-- Updates the visual elements
function RealmWealth:UpdateRealmWealthTotal()
	
	local amount = self:GetTotalRealmCharactersGold(GameLib.GetRealmName(), GameLib.GetPlayerUnit():GetFaction())
	
	self.realmtotal = amount
	
	local otherFactionAmount = 0
	
	if GameLib.GetPlayerUnit():GetFaction() == Unit.CodeEnumFaction.ExilesPlayer then
		otherFactionAmount = self:GetTotalRealmCharactersGold(GameLib.GetRealmName(), Unit.CodeEnumFaction.DominionPlayer)
	else
		otherFactionAmount = self:GetTotalRealmCharactersGold(GameLib.GetRealmName(), Unit.CodeEnumFaction.ExilesPlayer)
	end
	
	-- update form elements
	
	-- HUD
	self.wndHUD:FindChild("WealthWindow"):SetAmount(self.realmtotal)
	
	self.wndMain:FindChild("MoneyForFaction"):SetAmount(self.realmtotal)
	self.wndMain:FindChild("MoneyForOtherFaction"):SetAmount(otherFactionAmount)
end

-- Returns the total gold for the characters of the given realm and faction
function RealmWealth:GetTotalRealmCharactersGold(realmname, faction)

	if self.realms[realmname] == nil then return 0 end

	local total = 0
	
	local realm = self.realms[realmname]
	
	for key, val in pairs(realm) do
		local character = self.realms[realmname][key]
		if character["faction"] == faction then
			total = total + character["wealth"]
		end
	end
	
	return total
end

-- Fires when player's currency changes (gains or loses money).
function RealmWealth:OnPlayerCurrencyChanged()
	
	if self.realms == nil then realms = {} end
	if self.realms[GameLib.GetRealmName()] == nil then
		self.realms[GameLib.GetRealmName()] = {}
	end
	
	self:UpdateCurrentCharacterWealth()
	self:UpdateRealmWealthTotal()
	
end


-- Save User Settings
function RealmWealth:OnSave(eType)
	-- save global settings (realm totals)
	if eType == GameLib.CodeEnumAddonSaveLevel.General then 
		
		local tSave = {}
		tSave["realms"] = self.realms
		tSave["version"] = self.version
		return tSave
	end
	
	-- save character specific settings (HUD location, etc)
	if eType ~= GameLib.CodeEnumAddonSaveLevel.Character then return end	
	
	local tSave = {}
	tSave["hudsettings"] = self.HUDSettings
	
	return tSave
end

-- Restore Saved User Settings
function RealmWealth:OnRestore(eType, t)
	if eType == GameLib.CodeEnumAddonSaveLevel.General then 
	
		local major = "1"
		local minor = "0"
	
		if t["version"] ~= nil then
			local ver = self:mysplit(t["version"],".")
			
			major = ver[0]
			minor = ver[1]
		end
	
		if t["version"] == nil or t["version"] ~= self.version then		
					
			if t["version"] == nil or minor == "0" or minor == "1" then
				-- handle older version of saves
				self.realms = {}
				
				rlms = t["realms"]
				-- no save data
				if rlms == nil then
					self.realms[GameLib.GetRealmName()] = {}
					return
				end
				
				-- update to new save format
				for key, rlm in pairs(rlms) do
					self.realms[key] = {}
					for name, money in pairs(rlms[key]) do
						self.realms[key][name] = {}
						self.realms[key][name]["wealth"] = money
					end
				end
				
				return
			end							
		end
	
		-- no update needed
		
		if t["realms"] ~= nil then
			self.realms = t["realms"]
		else
			self.realms = {}
		end
	
		
		if self.realms[GameLib.GetRealmName()] == nil then
			self.realms[GameLib.GetRealmName()] = {}
		end
		
		return
	end
	
	if eType ~= GameLib.CodeEnumAddonSaveLevel.Character then return end		
	
	-- load character-specific HUD settings
	if t["hudsettings"] ~= nil then
		self.HUDSettings = t["hudsettings"]	
		
		if self.HUDSettings["show"] == nil then
			self.HUDSettings["show"] = true 
		end
		
		if self.HUDSettings["show_text"] == nil then
			self.HUDSettings["show_text"] = true
		end		
		
	end

end

function RealmWealth:mysplit(inputstr, sep)
        if sep == nil then
                sep = "%s"
        end
        local t={} ; i=1
        for str in string.gmatch(inputstr, "([^"..sep.."]+)") do
                t[i] = str
                i = i + 1
        end
        return t
end

-----------------------------------------------------------------------------------------------
-- RealmWealthForm Functions
-----------------------------------------------------------------------------------------------
-- when the OK button is clicked
function RealmWealth:OnOK()
	self.wndMain:Close() -- hide the window
end

-- when the Cancel button is clicked
function RealmWealth:OnCancel()
	self.wndMain:Close() -- hide the window
end

function RealmWealth:Button_ShowHUD( wndHandler, wndControl, eMouseButton )
	self.HUDSettings["show"] = self.wndMain:FindChild("Button_ShowHUD"):IsChecked()
	
	if self.HUDSettings["show"] == true then
		self.wndHUD:Invoke()
	else
		self.wndHUD:Close()
	end
end

function RealmWealth:Button_ShowHUDTitle( wndHandler, wndControl, eMouseButton )

	self.HUDSettings["show_text"] = self.wndMain:FindChild("Button_ShowHUDTitle"):IsChecked()
	
	if self.HUDSettings["show_text"] == true then
		self.wndHUD:SetText("Realm Wealth:")
	else
		self.wndHUD:SetText("")
	end
end

---------------------------------------------------------------------------------------------------
-- RealmWealthUI Functions
---------------------------------------------------------------------------------------------------

function RealmWealth:HUDWindowMove( wndHandler, wndControl, nOldLeft, nOldTop, nOldRight, nOldBottom )
	local wleft, wtop = self.wndHUD:GetPos()
	self.HUDSettings["position"] = { left = wleft, top = wtop }
	
end

-----------------------------------------------------------------------------------------------
-- RealmWealth Instance
-----------------------------------------------------------------------------------------------
local RealmWealthInst = RealmWealth:new()
RealmWealthInst:Init()
